/*
Copyright (C) 2014 Graham Jones, www.indriid.com
*/


package stacey.util;

import beast.core.BEASTObject;
import beast.core.Description;
import beast.evolution.alignment.TaxonSet;
import stacey.util.BitUnion;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Created by Graham Jones on 27/08/2014.
 */


@Description("Represents a TaxonSet of TaxonSets of Taxons")

/*
A Bindings stores the information in a TaxonSet of TaxonSets of Taxons,
and provides various access methods to this info.
*/
public class Bindings {
    private final Map<String, Integer> taxon2index = new HashMap<String, Integer>();
    private final Map<String, Integer> smcID2index = new HashMap<String, Integer>();
    private final int [] alltaxa2smc;
    private final List<String> smcIDs;
    private final int [] smcSizes;
    private final BitUnion [] smcTipUnions;



    // TODO. implement this sensible-looking check pasted from SpeciesTreePrior
    // where?
    // make sure the m_taxonSet is a set of taxonsets
    // HACK to make Beauti initialise: skip the check here
    // for (Taxon taxon : m_taxonSet.get().m_taxonset.get()) {
    //			if (!(taxon instanceof TaxonSet)) {
    //				throw new Exception("taxonset should be sets of taxa only, not individual taxons");
    //			}
    //
    //
    // }



    public Bindings(TaxonSet smcs) {
        int smcCount = smcs.getTaxonCount();
        smcIDs = smcs.asStringList();
        smcTipUnions = new BitUnion[smcCount];
        // make tip unions for smc tree
        for (int i = 0; i < smcCount; i++) {
            smcTipUnions[i] = new BitUnion(smcCount);
            smcTipUnions[i].insert(i);
        }
        for (int i = 0; i < smcCount; i++) {
            smcID2index.put(smcIDs.get(i), i);
        }


        // count all taxa
        smcSizes = new int[smcCount];
        int allTaxaCount = 0;
        for (int i = 0; i < smcCount; i++) {
            TaxonSet smc = (TaxonSet) smcs.taxonsetInput.get().get(i);
            smcSizes[i] = smc.getTaxonCount();
            allTaxaCount += smc.getTaxonCount();
        }
        // gather all taxa
        String[] alltaxa = new String[allTaxaCount];
        int ij = 0;
        for (int i = 0; i < smcCount; i++) {
            TaxonSet smc = (TaxonSet) smcs.taxonsetInput.get().get(i);
            for (int j = 0; j < smc.getTaxonCount(); j++, ij++) {
                alltaxa[ij] = smc.asStringList().get(j);
            }
        }
        assert ij == alltaxa.length;
        // set up map of all taxa to indices
        for (int i = 0; i < alltaxa.length; i++) {
            taxon2index.put(alltaxa[i], i);
        }

        // set up array mapping all-taxa index to smc index (for unions)
        alltaxa2smc = new int[alltaxa.length];
        ij = 0;
        for (int i = 0; i < smcs.getTaxonCount(); i++) {
            TaxonSet smc = (TaxonSet) smcs.taxonsetInput.get().get(i);
            for (int j = 0; j < smc.getTaxonCount(); j++, ij++) {
                alltaxa2smc[ij] = i;
            }
        }
        assert ij == alltaxa.length;
    }


    public int smcTreeTipCount() {
        return smcIDs.size();
    }

    public BitUnion tipUnionFromSmcID(String id) {
        int smc = smcID2index.get(id);
        return smcTipUnions[smc];
        /*
        int spi = smcId2index(tx);
        BitUnion x = new BitUnion(smcIDs.size());
        x.insert(spi);
        return x;*/
    }


    public BitUnion tipUnionFromAllTaxaID(String tx) {
        assert tx != null;
        assert smcTipUnions != null;
        int ati = allTaxaID2index(tx);
        int smc = allTaxaIndex2smcIndex(ati);
        return smcTipUnions[smc];
    }



    int allTaxaID2index(String allTaxaID) {
        assert taxon2index != null;
        assert allTaxaID != null;
        if (!taxon2index.containsKey(allTaxaID)) {
            System.out.println("Cannot find taxon ID '" + allTaxaID + "'.\n" +
                    "Make sure Taxon names and Species/Population names are distinct in Taxon tab in Beauti.\n");
            System.exit(1);
        }
        assert taxon2index.containsKey(allTaxaID);
        int atid = taxon2index.get(allTaxaID);
        assert atid >= 0;
        return atid;
    }

    int allTaxaIndex2smcIndex(int ati) {
        return alltaxa2smc[ati];
    }



    public int nLineagesFromSMCID(String smcID) {
        int smc = smcID2index.get(smcID);
        return smcSizes[smc];
    }



}
